#!/usr/bin/perl

# $Id: lipm_megablast.pl 1497 2016-03-16 14:12:57Z gouzy $

# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    my($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../lib/");
    unshift(@INC, "$dirprg/../corelib/");
}

our $VERSION = do {my @r = (q$Rev: 1497 $ =~ /\d+/g); $r[0]};

use warnings;
use strict;
use IO::File;
use ParamParser;
use General;
use GeneralBioinfo;
use Carp;
use File::Which;
use Data::Dumper;

my $NCPUS        = 8;
my $NT_PER_SLICE = 1000000;
my $DEBUG        = &FALSE;
my $MGBLAST      = "-outfmt 6 -dust no -word_size 28 -evalue 1e-200";
my $MGBLASTPCI   = "-outfmt 6 -dust no -word_size 28 -perc_identity 95 -evalue 1e-200";

MAIN:
{
	my $cli = join(' ',@ARGV);
	my($raw_bl) = $cli =~ / (t*blast\w\s.+)/;
	&Usage() if ( ! defined($raw_bl) );
	#print "$raw_bl\n";
    my $tmpsubst=$raw_bl;
	$tmpsubst=~s/\+/\\+/g;
    $cli =~ s,$tmpsubst,,;
	@ARGV=split(' ',$cli);
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'ncpus=i','nt_per_slice=i','mg','img');
	my @a_opt=('query','db','out');
	foreach my $opt (@a_opt)
	{
		my($file)=$raw_bl=~/\-$opt\s+(\S+)/;
		$o_param->SetUnlessDefined($opt,$file);
	}
	if ( $o_param->IsDefined('mg') )
	{
		$raw_bl .= " $MGBLAST";
	}
	elsif ( $o_param->IsDefined('img') )
	{
		$raw_bl .= " $MGBLASTPCI";
	}
	#$o_param->Print();
	$o_param->AssertFileExists('db','query');
	$o_param->AssertDefined('out');
	$o_param->SetUnlessDefined('ncpus',$NCPUS);
	$o_param->SetUnlessDefined('nt_per_slice',$NT_PER_SLICE);
	my ($blastprg) = $raw_bl =~ /^(t*blast[npx])/;
	&Usage() if ( ! defined($blastprg) );
	foreach my $prg ($blastprg,'lipm_fastasplitter.pl','lipm_smp.pl')
	{
		my $exe_path = which($prg);
		if (! defined($exe_path) )
		{
			Carp::croak("ERROR - $prg is NOT in the path, or NOT installed");
		}
	}

	my $out   = $o_param->Get('out');
	my $in    = $o_param->Get('query');
	my $ncpus = $o_param->Get('ncpus');

	my $wrkdir = ".";
	if ( $out =~ /\// )
	{
		($wrkdir) = $out =~ /(.+)\/.+/;
	}
	$wrkdir .= "/lipm_blast.$$.tmpdir";
	mkdir($wrkdir);
	my $tmpfile = "$wrkdir/lipm_blast.tmp.$$.".time;
	my $nt_per_slice = $o_param->Get('nt_per_slice');

	my $cde = "lipm_fastasplitter.pl --in $in --outprefix $tmpfile.split --maxlength_per_slice $nt_per_slice";
	&System($cde,$DEBUG);

	my @a_ls = `ls $tmpfile.split*`;
	chomp(@a_ls);
	exit(1) if (! scalar(@a_ls) );
	my $f_out = &GetStreamOut("$tmpfile.cmd");
	foreach my $tmp (@a_ls)
	{
		my $template = $raw_bl;
		$template =~ s/(\-query\s+$in)/-query $tmp/;
		$template =~ s/(\-out\s+$out)/-out $tmp.out/;
		print $f_out "$template\n";
	}
	$f_out->close;
	$cde = "lipm_smp.pl --in $tmpfile.cmd --ncpus $ncpus";
	&System($cde,$DEBUG);

	@a_ls = `ls $tmpfile.split*.out`;
	chomp(@a_ls);
	$cde = "cat ".join(' ',@a_ls)." > $out";
	&System($cde,$DEBUG);

	@a_ls = `ls $tmpfile.split*`;
	chomp(@a_ls);
	&System("rm $tmpfile.cmd ".join(' ',@a_ls),$DEBUG);
}

sub System
{
my($cde,$debug)=@_;

	print STDERR "$cde\n" if ($debug);
	system($cde);
}

sub Usage
{
print STDOUT <<END;

Usage:

$0 --ncpus integer --nt_per_slice integer [t]blast[nxp] ......

Mandatory: options: -query, -db, -out

Optional :
	--ncpus        integer  [$NCPUS]
	--nt_per_slice integer  [$NT_PER_SLICE]
	--mg                    flag to set blastn search at $MGBLAST
	--img                   flag to set blastn search at $MGBLASTPCI
END
exit(1);
}

