package BiomobyUtils;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

BiomobyUtils

=head1 DESCRIPTION

Utilities library

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 


=head1 METHODS

=cut

use strict;

use ParamParser;
use MOBY::Client::Central;
use Carp;
use vars qw(@ISA @EXPORT);
require Exporter;


@ISA = qw(Exporter);

@EXPORT = qw(
  &Exit
  &SetMobyCentralENV
  &ReturnHashWhichCentralConf
  &AddToDispatcher
  &RemoveFromDispatcher
  &GetDispatcherServicesFile
  &InitializeServiceParameters
  &IsTheCorrectObject
  &getNodeContent
  &GetRegisteredInputArticleNames
  );

=head2 function GetRegisteredInputArticleNames

 Title        : GetRegisteredInputArticleNames

=cut

sub GetRegisteredInputArticleNames
{
    my $o_param                         = shift();
    my %h_registered_input_article_name = ();

    my @a_input_article_name_keys = $o_param->GetKeys('iar_?\d+_nm');

    foreach my $input_article_name_key (@a_input_article_name_keys)
    {
        my ($root_input_key) = ($input_article_name_key =~ /^(iar_?\d+)/);
        $h_registered_input_article_name{$o_param->Get($input_article_name_key)} = $root_input_key;
    }

    return \%h_registered_input_article_name;

}

=head2 procedure Exit

 Title        : Exit
 Usage        : &Exit($message)
 Prerequisite : none
 Function     : print a message on STDERR and exit program
 Returns      : none 
 Args         : $message Message to print out
 Globals      : none

=cut

sub Exit
{
    my $message = shift();
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = gmtime(time);
    print STDERR "\n$mday$mon$year $hour:$min:$sec \n$message\n\n";
    exit;
}

=head2 function SetMobyCentralENV

 Title        : SetMobyCentralENV
 Usage        : $mobycentral_name = &SetMobyCentralENV($rh_param,$central_conf)
 Prerequisite : none
 Function     : Parse config file to get MobyCentral(s) to connect to
                Update %ENV with correct MOBY_URI and MOBY_SERVER variables
 Returns      : $mobycentral_name: mobycentral name as defined in configuration file
 Args         : $rh_param: ParamParser structure
                $central_conf: central configuration file
 Globals      : none

=cut

sub SetMobyCentralENV
{
    my ($rh_param, $central_conf) = @_;

    $rh_param->SelectNameSpace($central_conf);
    $rh_param->Update($central_conf, 'A');
    $ENV{'MOBY_URI'}    = $rh_param->Get("mobycentral_uri");
    $ENV{'MOBY_SERVER'} = $rh_param->Get("mobycentral_url");
    my $mobycentral_name = $rh_param->Get("mobycentral_name");
    $rh_param->SelectNameSpace("");

    return $mobycentral_name;
}

=head2 function ReturnHashWhichCentralConf

 Title        : ReturnHashWhichCentralConf
 Usage        : $rh_which_central_conf = &ReturnHashWhichCentralConf($rh_param,\@a_central_conf)
 Prerequisite : none
 Function     : Parse central(s) config file to get configuration file associated to a mobycentral name 
 Returns      : $rh_which_central_conf: an hash ref
 Args         : $rh_param: ParamParser structure
                $ra_central_conf: reference to the list of configuration files
 Globals      : none

=cut

sub ReturnHashWhichCentralConf
{
    my ($rh_param, $ra_central_conf) = @_;

    my %h_which_central_conf = ();

    foreach my $central_conf (@{$ra_central_conf})
    {
        $rh_param->SelectNameSpace($central_conf);
        $rh_param->Update($central_conf, 'A');
        my $mobycentral_name = $rh_param->Get("mobycentral_name");
        $h_which_central_conf{$mobycentral_name} = $central_conf;
        $rh_param->SelectNameSpace("");
    }

    return \%h_which_central_conf;

}

=head2 function AddToDispatcher

 Title        : AddToDispatcher
 Usage        : &AddToDispatcher($dispatcher_url,$service_name)
 Prerequisite : none
 Function     : Add service name to the file used by dispatcher CGI to 
 		construct hash of services available
 Returns      : undef if OK
 Args         : $dispatcher_url: CGI used by service as dispatcher
 		$service_name: service name
		$module_name: module containing your Service sub used in LIPMServices.pm BEGIN BLOCK
 Globals      : none

=cut

sub AddToDispatcher
{
    (my $dispatcher_url, my $service_name, my $module_name, my $registry_name, my $module_parent_directory) = (@_);

    if (!(defined $module_parent_directory))
    {

        #$module_name = $module_parent_directory . '::' . $module_name;
        $module_parent_directory = '';
    }
    my $services_file = &GetDispatcherServicesFile($dispatcher_url);

    if (-e $services_file)
    {
        open(SERVICES, "$services_file") || &Exit("$! :$services_file");
        while (my $service_line = <SERVICES>)
        {
            chomp $service_line;

            #my ($service, $module) = split('\|', $service_line);
            my ($service, $lib, $module,$registry) = split('\|', $service_line);
            
		
            if ($service eq $service_name)
            {
		if (((defined $registry) && ($registry ne '') && ($registry eq $registry_name)) ||(!defined $registry) ||($registry eq ''))
		{
                #&Exit("ERROR : $service_name is already in your dispatcher $services_file!");
                print STDERR
                  "\n\nWARNING $service_name is already in your dispatcher $dispatcher_url for $registry_name registry ==> $service_line\n\n";
                return;
		}
            }
        }
        close SERVICES;
    }

    open(SERVICES, ">> $services_file") || &Exit("$! :$services_file");
    print SERVICES "$service_name\|$module_parent_directory\|$module_name\|$registry_name\n";
    close SERVICES;
    print STDOUT "\n$service_name added to $services_file for registry $registry_name\n\n";
    return;
}

=head2 function RemoveFromDispatcher

 Title        : RemoveFromDispatcher
 Usage        : &RemoveFromDispatcher($dispatcher_url,$service_name)
 Prerequisite : none
 Function     : Remove service name from the file used by dispatcher CGI to 
 		construct hash of services available
 Returns      : undef if OK
 Args         : $dispatcher_url: CGI used by service as dispatcher
 		$service_name: service name
 Globals      : none

=cut

sub RemoveFromDispatcher
{
    (my $dispatcher_url, my $service_name, my $registry_name) = @_;

    my $services_file = &GetDispatcherServicesFile($dispatcher_url);

    my @a_services_to_keep = ();
    if (-e $services_file)
    {
        open(SERVICES, "$services_file") || &Exit("$! :$services_file");

        while (my $service_line = <SERVICES>)
        {
            chomp $service_line;

            my ($service, $module,$path, $registry) = split('\|', $service_line);

            chomp $service;
            if ($service eq $service_name)
            {
		if ((defined $registry) && ($registry ne '') && ($registry ne $registry_name))
            	{
			push(@a_services_to_keep, $service_line);
            	}
		
            }
            elsif ($service ne $service_name)
            {
                push(@a_services_to_keep, $service_line);
            }
        }
        close SERVICES;

    }

    open(SERVICES, "> $services_file") || &Exit("$! :$services_file");
    foreach my $service_name (@a_services_to_keep)
    {
        print SERVICES "$service_name\n";
    }
    close SERVICES;
    print STDOUT "\n$service_name removed from $services_file\n\n";
    return;

}

=head2 function GetDispatcherServicesFile

 Title        : GetDispatcherServicesFile
 Usage        : &GetDispatcherServicesFile($dispatcher_url)
 Prerequisite : none
 Function     : From a dispatcher URL, get the file name used by the
 		dispatcher CGI to build hash of available services
 Returns      : $services_file: file containing services names
 Args         : $dispatcher_url: CGI used by service as dispatcher
 Globals      : none

=cut

sub GetDispatcherServicesFile
{
    (my $dispatcher_url) = @_;

    my $dispatcher_cfg = $ENV{PATH_WS_BIOMOBY} . '/cfg/dispatcher/dispatcher.cfg';

    if (!-e $dispatcher_cfg)
    {
        print STDERR "$dispatcher_cfg does not exist !\n\n";
        exit;
    }

    my $o_param_dispatcher = New ParamParser($dispatcher_cfg);

    $o_param_dispatcher->SetSubstitution('%p', $ENV{PATH_WS_BIOMOBY});
    $o_param_dispatcher->SetSubstitution('%u', $ENV{URL_WS_BIOMOBY});
    $o_param_dispatcher->SetBehaviour('use_substitution_table');

    my @a_dispatcher_url_tags = $o_param_dispatcher->GetKeys('dispatcher\d+_url');

    my $rh_param_namespace = '';

    foreach my $dispatcher_url_tag (@a_dispatcher_url_tags)
    {
        if ($o_param_dispatcher->Get($dispatcher_url_tag) =~ /\%u/)
        {
            my $value = $o_param_dispatcher->Get($dispatcher_url_tag);
            my $url   = $ENV{URL_WS_BIOMOBY};
            $value =~ s/\%u/$url/g;
            $o_param_dispatcher->Set($dispatcher_url_tag, $value);
        }

        if ($o_param_dispatcher->Get($dispatcher_url_tag) eq $dispatcher_url)
        {
            ($rh_param_namespace) = ($dispatcher_url_tag =~ /(dispatcher\d+_)/);
        }
    }

    if ($rh_param_namespace eq '')
    {
        print STDERR "$dispatcher_url unknown !\n\n";

    }

    $o_param_dispatcher->SelectNameSpace($rh_param_namespace);
    my $services_file = $o_param_dispatcher->Get("services");
    $services_file =~ s/\%p/$ENV{PATH_WS_BIOMOBY}/;
    return $services_file;
}

################################################
#
# DEVELOPPEMENT DE SERVICES
#
################################################

=head2 function InitializeServiceParameters

 Title        : InitializeServiceParameters
 Usage        : &InitializeServiceParameters($rh_param)
 Prerequisite : none
 Function     : From service configuration file, build a service 
 		parameters strucure initialized with default values
 Returns      : \%h_param_service: Hash ref for service parameters strucure (param1 => default, param2 => default, ...)
 Args         : $rh_param: ParamParser Structure
 Globals      : none

=cut

sub InitializeServiceParameters
{
    my $rh_param = shift();

    my %h_param_service = ();

    my @a_sar_tags = $rh_param->GetKeys('sar\d+_name');
    foreach my $sar_tag (@a_sar_tags)
    {
        (my $name_space) = ($sar_tag =~ /(sar\d+_)name/);
        $rh_param->SelectNameSpace($name_space);
        $h_param_service{$rh_param->Get("name")} = $rh_param->Get("default");
        $rh_param->SelectNameSpace('');
    }

    return \%h_param_service;
}

=head2 function IsTheCorrectObject

 Title        : IsTheCorrectObject
 Usage        : &IsTheCorrectObject($tested_nodename,$object_for_service)
 Prerequisite : none
 Function     : Test if a XML nodeName is the one requested
 		Usefull for data extraction (is the node tested a FASTA node ?)
		This function gives priority to Canadian Ontology server, then Local.
 Returns      : boolean
 Args         : $tested_nodename: XML node tested
 		$object_for_service: requested node
 Globals      : none

=cut

sub IsTheCorrectObject
{
    my ($tested_node, $object_for_service, $central_instance) = (@_);
    my $tested_nodename = $tested_node;

    $tested_nodename =~ s/^moby://;
    if ($tested_nodename eq $object_for_service)
    {
        return 1;
    }
    elsif (defined $central_instance)
    {
        my $isa = 0;
        $isa = &_ISA_by_carrere($central_instance, $tested_nodename, $object_for_service);
        if ($isa)
        {
            return $isa;
        }
    }
    else
    {

        #La reference de l'ontologie des objets est sur le central canadien
        #$ENV{'MOBY_URI'}    = 'http://moby.ucalgary.ca/MOBY/Central';
        #$ENV{'MOBY_SERVER'} = 'http://moby.ucalgary.ca/moby/MOBY-Central.pl';
		$ENV{'MOBY_URI'}    = 'http://moby.toulouse.inra.fr/MOBY/Central';
        $ENV{'MOBY_SERVER'} = 'http://moby.toulouse.inra.fr/cgi/MOBY-Central.pl';
		$ENV{'MOBY_ONTOLOGYSERVER'} = 'http://moby.toulouse.inra.fr/cgi/OntologyServer.cgi';
		
        my $central_instance = MOBY::Client::Central->new();
        my $isa              = 0;

        $isa = &_ISA_by_carrere($central_instance, $tested_nodename, $object_for_service);

        if ($isa)
        {
            return $isa;
        }
        else
        {

            #on teste sur l'annuaire local
            #$ENV{'MOBY_URI'}    = 'http://genopole.toulouse.inra.fr/MOBY/Central';
            #$ENV{'MOBY_SERVER'} = 'http://genopole.toulouse.inra.fr/cgi-bin/MOBY-Central.pl';
            #$central_instance = MOBY::Client::Central->new();
            #$isa              = &_ISA_by_carrere($central_instance, $tested_nodename, $object_for_service);

        }

        return $isa;
    }

}

=head2 function _ISA_by_carrere

 Title        : _ISA_by_carrere
 Usage        : &_ISA_by_carrere($central_instance, $tested_nodename,$object_for_service)
 Prerequisite : none
 Function     : Test if an object ISA another (recursively).
 		I use this home-made sub because ISA sub from PERL-API
		use only the Canadian OntologyServer.
		So if our object is not registered in Canadian Central, ISA 
		doesn't work ...
 Returns      : boolean
 Args         : $central_instance: MOBY::Client::Central->new object
 		$tested_nodename: XML node tested
 		$object_for_service: requested node
 Globals      : none

=cut

sub _ISA_by_carrere
{
    my ($central_instance, $tested_object, $object_for_service) = (@_);

    my $rs_lsid_root     = 'urn:lsid:biomoby.org:objectrelation:';
    my $object_lsid_root = 'urn:lsid:biomoby.org:objectclass:';

    $tested_object =~ s/^$object_lsid_root//;

    $object_for_service =~ s/^$object_lsid_root//;

    my $rh_rs = $central_instance->Relationships(objectType => $tested_object, Relationships => ['ISA']);

    my @a_rs = keys %{$rh_rs};
    my $rs   = shift @a_rs;

    if ($rs eq '')
    {
        return 0;
    }

    my $rs_for_display = $rs;
    $rs_for_display =~ s/$rs_lsid_root//;

    #la fonction Relationships a chang�

    my $rh_related_object = shift(@{$rh_rs->{$rs}});
    my $related_object    = $rh_related_object->{term};

    $related_object =~ s/^$object_lsid_root//;

    if ($related_object eq $object_for_service)
    {
        return 1;
    }
    elsif ($related_object eq 'Object')
    {
        return 0;
    }
    else
    {
        &_ISA_by_carrere($central_instance, $related_object, $object_for_service);
    }

}

=head2 function GetNodeContent

	

=cut

sub GetNodeContent
{
    my $xml_dom = shift();
    my $content = '';
    foreach my $subnode ($xml_dom->getChildNodes)
    {
        $content .= $subnode->getData;
    }
    return $content;
}

sub GetParentObject
{
    my ($tested_object) = shift;

    #$ENV{'MOBY_URI'}    = 'http://moby.ucalgary.ca/MOBY/Central';
    #$ENV{'MOBY_SERVER'} = 'http://moby.ucalgary.ca/moby/MOBY-Central.pl';
	$ENV{'MOBY_URI'}    = 'http://moby.toulouse.inra.fr/MOBY/Central';
    $ENV{'MOBY_SERVER'} = 'http://moby.toulouse.inra.fr/cgi/MOBY-Central.pl';
	$ENV{'MOBY_ONTOLOGYSERVER'} = 'http://moby.toulouse.inra.fr/cgi/OntologyServer.cgi';
    my $central_instance = MOBY::Client::Central->new();

    my $rs_lsid_root     = 'urn:lsid:biomoby.org:objectrelation:';
    my $object_lsid_root = 'urn:lsid:biomoby.org:objectclass:';

    $tested_object =~ s/^$object_lsid_root//;


    my $rh_rs = $central_instance->Relationships(objectType => $tested_object, Relationships => ['ISA']);

    my @a_rs = keys %{$rh_rs};
    my $rs   = shift @a_rs;

    if ($rs eq '')
    {
        return 0;
    }

    my $rs_for_display = $rs;
    $rs_for_display =~ s/$rs_lsid_root//;

    #la fonction Relationships a change

    my $rh_related_object = shift(@{$rh_rs->{$rs}});
    my $related_object    = $rh_related_object->{term};

    $related_object =~ s/^$object_lsid_root//;

    return $related_object;
    
}

sub IsCentralAlive
{
	my $url = 'http://moby.ucalgary.ca';
	my $timeout = 3;
	use LWP::Simple qw($ua);
	$LWP::Simple::ua->timeout($timeout);
	my $cgi_ok = LWP::Simple::get($url);
	my $alive = 0;
	$alive = 1 if (defined $cgi_ok);
	return $alive;
}
1;
